#include <plib.h>
#include "LCD.h"
#define CLK LATGbits.LATG15         // CLK falling edge sends data to controller

void LCDWriteChar(char c);          // write a char to the LCD's cursor position
void LCDcommand(int command, int d7, int d6, int d5, int d4, int d3, int d2, int d1, int d0);
void wait();                        // make sure clock pulses long enough

void LCDSetup() {
  TRISGbits.TRISG12 = 0;            // G12 is output to LCD RS pin (cmd or data)
  TRISGbits.TRISG15 = 0;            // G15 is output to LCD CLK line
  TRISECLR = 0xFF;                  // RE0..7 are outputs to LCD D0..7
  LCDcommand(0, 0,0,1,1,1,0,0,0);   // initialize 2 lines
  LCDcommand(0, 0,0,0,0,0,0,0,1);   // clear screen
  LCDcommand(0, 0,0,0,0,0,1,1,0);   // cursor moves right
  LCDcommand(0, 0,0,0,0,1,1,0,0);   // restore screen
}

void LCDWriteChar(char c) {         // send 8 bits of data for char c
  LCDcommand(1, c>>7&1, c>>6&1, c>>5&1, c>>4&1, c>>3&1, c>>2&1, c>>1&1, c&1);
}

void LCDWriteString(char *str, int row, int col) {
  row--; col--;                     // LCD uses rows 0-1, cols 0-15
  LCDcommand(0,1,row,0,0,col>>3&1,col>>2&1,col>>1&1,col&1);
  while(*str) LCDWriteChar(*str++); // increment string pointer after char sent
}

void LCDClear(int line) {
  switch(line) {
    case 1:                         // clear line 1
        LCDWriteString("                ", 1, 1);
        break;
    case 2:                         // clear line 2
        LCDWriteString("                ", 2, 1);
        break;
    default:                        // clear both lines
        LCDcommand(0,0,0,0,0,0,0,0,1);
  }
}

void LCDcommand(int command, int d7, int d6, int d5, int d4, int d3, int d2, int d1, int d0) {
  LATGbits.LATG12 = command;        // 0 for command, 1 for data
  LATECLR = 0xFF;                   // clear bits E0-7, then write data to them
  LATE = LATE | d0 | (d1<<1) | (d2<<2) | (d3<<3) | (d4<<4) | (d5<<5) | (d6<<6) | (d7<<7);
  CLK = 1; wait();                  // set CLK high and wait
  CLK = 0; wait();                  // data sent to HD44780 on CLK falling edge
}

void wait() {                       // ensure pulse is long enough (device dep)
  int i = 0;
  for (; i<10000; i++);
}
